/*******************************************************************
                      C O P Y R I G H T  (c) 1998
        F O C U S     A U T O M A T I O N   S Y S T E M S   I N C.
                          All Rights Reserved.
 -------------------------------------------------------------------

 base.c - Base window functions.

 -------------------------------------------------------------------
 *******************************************************************
 * $Source: $
 * $Date: $
 * $Revision: $
 *******************************************************************
*/

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <assert.h>
#include <sys/name.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "proto.h"
#include "config.h"
#include "input_group.h"
#include "args.h"
#include "msg.h"


static int register_name( char *name )
{
    int        error = 0;
    static int name_id;

    if ( name_id > 0 )
    {
        qnx_name_detach( 0, name_id );
        name_id = 0;
    }

    if ( name )
    {
        name_id = qnx_name_attach( 0, name );

        if ( name_id < 0 )
        {
            name_id = 0;
            error = -1;
        }
    }

    return ( error );
}


static int rects_intersect( const PhPoint_t *pos1, const PhPoint_t *size1, 
                            const PhRect_t *rect2 )
{
    int  intersect   = Pt_FALSE;
    int  x_intersect = Pt_FALSE;
    int  y_intersect = Pt_FALSE;

    if ( pos1->x                <= rect2->ul.x &&
         pos1->x + size1->x - 1 >= rect2->ul.x    )
    {
        x_intersect = Pt_TRUE;
    }
    else
    if ( pos1->x >= rect2->ul.x &&
         pos1->x <= rect2->lr.x    )
    {
        x_intersect = Pt_TRUE;
    }

    if ( pos1->y                <= rect2->ul.y &&
         pos1->y + size1->y - 1 >= rect2->ul.y    )
    {
        y_intersect = Pt_TRUE;
    }
    else
    if ( pos1->y >= rect2->ul.y &&
         pos1->y <= rect2->lr.y    )
    {
        y_intersect = Pt_TRUE;
    }

    if ( x_intersect && y_intersect )
    {
        intersect = Pt_TRUE;
    }

    return ( intersect );
}


static void check_avoidance_rect(       PhPoint_t *pos,
                                  const PhPoint_t *size,
                                  const PhRect_t  *input_group )
{
    int  left_space, right_space, top_space, bottom_space;
    int  border = 10;

    if ( AvoidAreaSpecified &&
         rects_intersect( pos, size, &AvoidArea ) )
    {
        left_space  = AvoidArea.ul.x    - input_group->ul.x;
        right_space = input_group->lr.x - AvoidArea.lr.x;

        top_space    = AvoidArea.ul.y    - input_group->ul.y;
        bottom_space = input_group->lr.y - AvoidArea.lr.y;

        if ( left_space > right_space )
        {
            pos->x = AvoidArea.ul.x - size->x - border;
        }
        else
        {
            pos->x = AvoidArea.lr.x + border;
        }

        if ( top_space > bottom_space )
        {
            pos->y = AvoidArea.ul.y - size->y - border;
        }
        else
        {
            pos->y = AvoidArea.lr.y + border;
        }
    }

    return;
}


int window_manager_callback( PtWidget_t       *widget,
                             ApInfo_t         *apinfo,
                             PtCallbackInfo_t *cbinfo )
{
    PhWindowEvent_t  *event;
    PhRect_t          input_group;

    widget = widget, apinfo = apinfo, cbinfo = cbinfo;

    if ( cbinfo->reason == Pt_CB_WINDOW )
    {
        event = (PhWindowEvent_t *)cbinfo->cbdata;

        if ( event->event_f & Ph_WM_MOVE )
        {
            get_input_group_bounds( &input_group );

            WindowOrigin.x = event->pos.x - input_group.ul.x;
            WindowOrigin.y = event->pos.y - input_group.ul.y;
        }
    }

    return( Pt_CONTINUE );
}


/*
--------------------------------------------------------------------------
setup_base() Photon callback;

This function is called as both a pre-realize and post-realize setup
callback.  Its main responsibility is the positioning of the window on
screen.  This would be best done as a pre-realize callback only, since
moving the dialog in a post-realize callback causes visible flicker.
Unfortunately, if the dialog position is specified on the command line
via '-x <n> -y <n>', the pre-realize function is called before the window
is positioned.  Therefore, it must be called again as a post-realize
function to correctly handle this case.
--------------------------------------------------------------------------
*/

int setup_base( PtWidget_t       *link_instance,
                ApInfo_t         *apinfo,
                PtCallbackInfo_t *cbinfo )
{
    PtArg_t     args[10];
    PhRect_t    input_group, extent;
    PhPoint_t   pos, size, overlap;
    int         nargs, error;
    unsigned   *prender_flags;
    int         left_border, right_border, top_border, bottom_border;
    static int  first_time = Pt_TRUE;

    apinfo = apinfo, cbinfo = cbinfo;

    if ( first_time )
    {
        error = register_name( REGISTERED_NAME );

        PtAppAddInput( NULL, 0, msg_handler, 0 );

        if ( error )
        {
            perror( REGISTERED_NAME );
            exit( EXIT_FAILURE );
        }

        first_time = Pt_FALSE;
    }

    get_input_group_bounds( &input_group );

    PtExtentWidget( link_instance );
    PtWidgetExtent( link_instance, &extent );

    nargs = 0;

    PtSetArg( &args[nargs++], Pt_ARG_WINDOW_RENDER_FLAGS, &prender_flags, 0 );

    PtGetResources( link_instance, nargs, &args );

    PtFrameSize( *prender_flags, 0,
                 &left_border, &right_border,
                 &top_border,  &bottom_border );

    size.x = extent.lr.x - extent.ul.x + 1 + left_border + right_border;
    size.y = extent.lr.y - extent.ul.y + 1 + top_border  + bottom_border;

    if ( LocationSpecified )
    {
        pos = Location;
    }
    else
    {
        pos.x = WindowOrigin.x + input_group.ul.x;
        pos.y = WindowOrigin.y + input_group.ul.y;
    }

    check_avoidance_rect( &pos, &size, &input_group );

    overlap.x = (pos.x + size.x - 1) - input_group.lr.x;
    overlap.y = (pos.y + size.y - 1) - input_group.lr.y;

    if ( overlap.x > 0 )
        pos.x -= overlap.x;

    if ( overlap.y > 0 )
        pos.y -= overlap.y;

    if ( pos.x < input_group.ul.x )
        pos.x = input_group.ul.x;

    if ( pos.y < input_group.ul.y )
        pos.y = input_group.ul.y;

    nargs = 0;
    PtSetArg( &args[nargs++], Pt_ARG_POS, &pos, 0 );

    PtSetResources( ABW_base, nargs, &args );

    WindowOrigin.x = pos.x - input_group.ul.x;
    WindowOrigin.y = pos.y - input_group.ul.y;

    return( Pt_CONTINUE );
}


int base_unrealized( PtWidget_t       *widget,
                     ApInfo_t         *apinfo,
                     PtCallbackInfo_t *cbinfo )
{
    widget = widget, apinfo = apinfo, cbinfo = cbinfo;

    register_name( NULL );

    return( Pt_CONTINUE );
}

